@@ -1,20 +1,16 @@
-from __future__ import print_function
-
 import unittest
 import dbus
 import subprocess
 import os
 import time
 import re
+import shutil
 import sys
+import multiprocessing
 from datetime import datetime
 from enum import Enum
 from systemd import journal
-
-if sys.version_info.major == 3 and sys.version_info.minor >= 3:
-    from time import monotonic
-else:
-    from monotonic import monotonic
+from time import monotonic
 
 import gi
 gi.require_version('GUdev', '1.0')
@@ -23,6 +19,9 @@ from gi.repository import GUdev
 test_devs = None
 FLIGHT_RECORD_FILE = "flight_record.log"
 
+multiprocessing.set_start_method("fork")
+
+
 def run_command(command):
     res = subprocess.Popen(command, shell=True, stdout=subprocess.PIPE,
                            stderr=subprocess.PIPE)
@@ -34,6 +33,7 @@ def run_command(command):
         output = out.decode().strip()
     return (res.returncode, output)
 
+
 def get_call_long(call):
     def call_long(*args, **kwargs):
         """Do an async call with a very long timeout (unless specified otherwise)"""
@@ -95,52 +95,6 @@ def get_version():
     return (distro, version)
 
 
-def skip_on(skip_on_distros, skip_on_version="", reason=""):
-    """A function returning a decorator to skip some test on a given distribution-version combination
-
-    :param skip_on_distros: distro(s) to skip the test on
-    :type skip_on_distros: str or tuple of str
-    :param str skip_on_version: version of distro(s) to skip the tests on (only
-                                checked on distribution match)
-
-    """
-    if isinstance(skip_on_distros, str):
-        skip_on_distros = (skip_on_distros,)
-
-    distro, version = get_version()
-
-    def decorator(func):
-        if distro in skip_on_distros and (not skip_on_version or skip_on_version == version):
-            msg = "not supported on this distribution in this version" + (": %s" % reason if reason else "")
-            return unittest.skip(msg)(func)
-        else:
-            return func
-
-    return decorator
-
-def unstable_test(test):
-    """Decorator for unstable tests
-
-    Failures of tests decorated with this decorator are silently ignored unless
-    the ``UNSTABLE_TESTS_FATAL`` environment variable is defined.
-    """
-
-    def decorated_test(*args):
-        try:
-            test(*args)
-        except unittest.SkipTest:
-            # make sure skipped tests are just skipped as usual
-            raise
-        except Exception as e:
-            # and swallow everything else, just report a failure of an unstable
-            # test, unless told otherwise
-            if "UNSTABLE_TESTS_FATAL" in os.environ:
-                raise
-            print("unstable-fail: Ignoring exception '%s'\n" % e, end="", file=sys.stderr)
-
-    return decorated_test
-
-
 class DBusProperty(object):
 
     TIMEOUT = 5
@@ -161,7 +115,7 @@ class DBusProperty(object):
     def _update_value(self):
         self._value = self.obj.Get(self.iface, self.prop, dbus_interface=dbus.PROPERTIES_IFACE)
 
-    def _check(self, timeout, check_fn):
+    def _check(self, timeout, check_fn, poll_vg=None):
         for _ in range(int(timeout / 0.5)):
             try:
                 self._update_value()
@@ -171,24 +125,26 @@ class DBusProperty(object):
                 # ignore all exceptions -- they might be result of property
                 # not having the expected type (e.g. 'None' when checking for len)
                 pass
+            if poll_vg:
+                poll_vg.Poll(dbus_interface='org.freedesktop.UDisks2.VolumeGroup')
             time.sleep(0.5)
 
         return False
 
-    def assertEqual(self, value, timeout=TIMEOUT, getter=None):
+    def assertEqual(self, value, timeout=TIMEOUT, getter=None, poll_vg=None, msg=None):
         if getter is not None:
             check_fn = lambda x: getter(x) == value
         else:
             check_fn = lambda x: x == value
-        ret = self._check(timeout, check_fn)
+        ret = self._check(timeout, check_fn, poll_vg)
 
         if not ret:
             if getter is not None:
-                raise AssertionError('%s != %s' % (getter(self._value), value))
+                raise AssertionError('%s != %s%s' % (getter(self._value), value, ' (%s)' % msg if msg else ''))
             else:
-                raise AssertionError('%s != %s' % (self._value, value))
+                raise AssertionError('%s != %s%s' % (self._value, value, ' (%s)' % msg if msg else ''))
 
-    def assertNotEqual(self, value, timeout=TIMEOUT, getter=None):
+    def assertNotEqual(self, value, timeout=TIMEOUT, getter=None, msg=None):
         if getter is not None:
             check_fn = lambda x: getter(x) != value
         else:
@@ -197,9 +153,9 @@ class DBusProperty(object):
 
         if not ret:
             if getter is not None:
-                raise AssertionError('%s == %s' % (getter(self._value), value))
+                raise AssertionError('%s != %s%s' % (getter(self._value), value, ' (%s)' % msg if msg else ''))
             else:
-                raise AssertionError('%s == %s' % (self._value, value))
+                raise AssertionError('%s != %s%s' % (self._value, value, ' (%s)' % msg if msg else ''))
 
     def assertAlmostEqual(self, value, delta, timeout=TIMEOUT, getter=None):
         if getter is not None:
@@ -216,48 +172,47 @@ class DBusProperty(object):
                 raise AssertionError('%s is not almost equal to %s (delta = %s)' % (self._value,
                                                                                     value, delta))
 
-
-    def assertGreater(self, value, timeout=TIMEOUT):
+    def assertGreater(self, value, timeout=TIMEOUT, msg=None):
         check_fn = lambda x: x > value
         ret = self._check(timeout, check_fn)
 
         if not ret:
-            raise AssertionError('%s is not greater than %s' % (self._value, value))
+            raise AssertionError('%s is not greater than %s%s' % (self._value, value, ' (%s)' % msg if msg else ''))
 
-    def assertLess(self, value, timeout=TIMEOUT):
+    def assertLess(self, value, timeout=TIMEOUT, msg=None):
         check_fn = lambda x: x < value
         ret = self._check(timeout, check_fn)
 
         if not ret:
-            raise AssertionError('%s is not less than %s' % (self._value, value))
+            raise AssertionError('%s is not less than %s%s' % (self._value, value, ' (%s)' % msg if msg else ''))
 
-    def assertIn(self, lst, timeout=TIMEOUT):
+    def assertIn(self, lst, timeout=TIMEOUT, msg=None):
         check_fn = lambda x: x in lst
         ret = self._check(timeout, check_fn)
 
         if not ret:
             raise AssertionError('%s not found in %s' % (self._value, lst))
 
-    def assertNotIn(self, lst, timeout=TIMEOUT):
+    def assertNotIn(self, lst, timeout=TIMEOUT, msg=None):
         check_fn = lambda x: x not in lst
         ret = self._check(timeout, check_fn)
 
         if not ret:
-            raise AssertionError('%s unexpectedly found in %s' % (self._value, lst))
+            raise AssertionError('%s unexpectedly found in %s%s' % (self._value, lst, ' (%s)' % msg if msg else ''))
 
-    def assertTrue(self, timeout=TIMEOUT):
+    def assertTrue(self, timeout=TIMEOUT, msg=None):
         check_fn = lambda x: bool(x)
         ret = self._check(timeout, check_fn)
 
         if not ret:
-            raise AssertionError('%s is not true' % self._value)
+            raise AssertionError('%s is not true%s' % (self._value, ' (%s)' % msg if msg else ''))
 
-    def assertFalse(self, timeout=TIMEOUT):
+    def assertFalse(self, timeout=TIMEOUT, msg=None):
         check_fn = lambda x: not bool(x)
         ret = self._check(timeout, check_fn)
 
         if not ret:
-            raise AssertionError('%s is not false' % self._value)
+            raise AssertionError('%s is not false%s' % (self._value, ' (%s)' % msg if msg else ''))
 
     def assertIsNone(self, timeout=TIMEOUT):
         check_fn = lambda x: x is None
@@ -266,14 +221,14 @@ class DBusProperty(object):
         if not ret:
             raise AssertionError('%s is not None' % self._value)
 
-    def assertIsNotNone(self, timeout=TIMEOUT):
+    def assertIsNotNone(self, timeout=TIMEOUT, msg=None):
         check_fn = lambda x: x is not None
         ret = self._check(timeout, check_fn)
 
         if not ret:
-            raise AssertionError('unexpectedly None')
+            raise AssertionError('unexpectedly None%s' % ' (%s)' % msg if msg else '')
 
-    def assertLen(self, length, timeout=TIMEOUT):
+    def assertLen(self, length, timeout=TIMEOUT, msg=None):
         check_fn = lambda x: len(x) == length
         ret = self._check(timeout, check_fn)
 
@@ -281,17 +236,21 @@ class DBusProperty(object):
             if not hasattr(self._value, '__len__'):
                 raise AssertionError('%s has no length' % type(self._value))
             else:
-                raise AssertionError('Expected length %d, but %s has length %d' % (length,
-                                                                                   self._value,
-                                                                                   len(self._value)))
-    def assertContains(self, member, timeout=TIMEOUT):
+                raise AssertionError('Expected length %d, but %s has length %d%s' % (length,
+                                                                                     self._value,
+                                                                                     len(self._value,
+                                                                                     ' (%s)' % msg if msg else '')))
+    def assertContains(self, member, timeout=TIMEOUT, msg=None):
         check_fn = lambda x: member in x
         ret = self._check(timeout, check_fn)
 
         if not ret:
-            raise AssertionError('%s does not contain %s' % (self._value, member))
+            raise AssertionError('%s does not contain %s%s' % (self._value, member, ' (%s)' % msg if msg else ''))
+
 
 class UdisksTestCase(unittest.TestCase):
+    TIMEOUT = 5
+
     iface_prefix = None
     path_prefix = None
     bus = None
@@ -299,7 +258,6 @@ class UdisksTestCase(unittest.TestCase):
     distro = (None, None, None)       # (project, distro_name, version)
     no_options = dbus.Dictionary(signature="sv")
 
-
     @classmethod
     def setUpClass(self):
         self.iface_prefix = 'org.freedesktop.UDisks2'
@@ -313,15 +271,13 @@ class UdisksTestCase(unittest.TestCase):
         self.bus.call_async = get_call_long(self._orig_call_async)
         self.bus.call_blocking = get_call_long(self._orig_call_blocking)
         self.vdevs = test_devs
-        assert len(self.vdevs) > 3;
-
+        assert len(self.vdevs) > 3
 
     @classmethod
     def tearDownClass(self):
         self.bus.call_async = self._orig_call_async
         self.bus.call_blocking = self._orig_call_blocking
 
-
     def run(self, *args):
         record = []
         now = datetime.now()
@@ -369,12 +325,10 @@ class UdisksTestCase(unittest.TestCase):
             obj = self.get_object(obj)
         return dbus.Interface(obj, self.iface_prefix + iface_suffix)
 
-
     @classmethod
     def get_property(self, obj, iface_suffix, prop):
         return DBusProperty(obj, self.iface_prefix + iface_suffix, prop)
 
-
     @classmethod
     def get_property_raw(self, obj, iface_suffix, prop):
         res = obj.Get(self.iface_prefix + iface_suffix, prop, dbus_interface=dbus.PROPERTIES_IFACE)
@@ -418,13 +372,24 @@ class UdisksTestCase(unittest.TestCase):
             time.sleep(0.5)
         self.fail('Failed to unmount %s: %s' % (path, out))
 
+    def _conf_backup(self, conf_file):
+        """ Backup and restore @conf_file during cleanup. If @conf_file doesn't exist, it
+            will be removed during cleanup
+        """
+        if os.path.exists(conf_file):
+            # conf file exists -> backup and restore
+            contents = self.read_file(conf_file)
+            self.addCleanup(self.write_file, conf_file, contents)
+        else:
+            # conf file doesn't exist -> remove during cleanup
+            self.addCleanup(shutil.rmtree, conf_file, True)
+
     @classmethod
     def read_file(self, filename):
         with open(filename, 'r') as f:
             content = f.read()
         return content
 
-
     @classmethod
     def write_file(self, filename, content, ignore_nonexistent=False, binary=False):
         try:
@@ -528,17 +493,27 @@ class UdisksTestCase(unittest.TestCase):
         time.sleep(1)
 
     @classmethod
-    def assertHasIface(self, obj, iface):
-        obj_intro = dbus.Interface(obj, "org.freedesktop.DBus.Introspectable")
-        intro_data = obj_intro.Introspect()
-
-        for _ in range(20):
-            if ('interface name="%s"' % iface) in intro_data:
+    def assertHasIface(self, obj, iface, timeout=TIMEOUT):
+        for _ in range(timeout * 2):
+            obj_intro = dbus.Interface(obj, "org.freedesktop.DBus.Introspectable")
+            intro_data = obj_intro.Introspect()
+            if 'interface name="%s"' % iface in intro_data:
                 return
             time.sleep(0.5)
 
         raise AssertionError("Object '%s' has no interface '%s'" % (obj.object_path, iface))
 
+    def assertObjNotOnBus(self, obj_path, timeout=TIMEOUT):
+        objects = []
+        for _ in range(timeout * 2):
+            obj_mgr = self.get_object('')
+            objects = obj_mgr.GetManagedObjects(dbus_interface='org.freedesktop.DBus.ObjectManager')
+            if obj_path not in objects.keys():
+                return
+            time.sleep(0.5)
+
+        raise AssertionError("Object '%s' present on the object manager" % (obj_path))
+
     def assertStartswith(self, val, prefix):
         if not val.startswith(prefix):
             raise AssertionError("'%s' does not start with '%s'" % (val, prefix))
@@ -581,6 +556,7 @@ class FlightRecorder(object):
         # given as arguments was not handled
         return False
 
+
 class CmdFlightRecorder(FlightRecorder):
     """Flight recorder running a command and gathering its standard and error output"""
 
@@ -609,6 +585,7 @@ class CmdFlightRecorder(FlightRecorder):
         rec = '<<<<< ' + self._desc + ' >>>>>' + '\n' + out.decode() + '\n\n'
         self._store.append(rec)
 
+
 class JournalRecorder(FlightRecorder):
     """Flight recorder for gathering logs (journal)"""
 
@@ -624,20 +601,19 @@ class JournalRecorder(FlightRecorder):
         self._stopped = None
 
     def _start(self):
-        self._started = monotonic()
+        self._started = time.time()
 
     def _stop(self):
-        self._stopped = monotonic()
+        self._stopped = time.time()
 
     def _save(self):
         j = journal.Reader(converters={"MESSAGE": lambda x: x.decode(errors="replace")})
         j.this_boot()
-        j.seek_monotonic(self._started)
+        j.seek_realtime(self._started)
         journal_data = ""
 
         entry = j.get_next()
-        # entry["__MONOTONIC_TIMESTAMP"] is a tuple of (datetime.timedelta, boot_id)
-        while entry and entry["__MONOTONIC_TIMESTAMP"][0].seconds <= int(self._stopped):
+        while entry and int(entry["__REALTIME_TIMESTAMP"].timestamp()) <= int(self._stopped):
             if "_COMM" in entry and "_PID" in entry:
                 source = "%s[%d]" % (entry["_COMM"], entry["_PID"])
             else:
